require('dotenv');
const bodyParser = require('body-parser');
const autenticacaoRequest = require('../request/autenticacao-request');
const infoRequest = require('../request/info-request');
const jwt = require('jsonwebtoken');

module.exports = ({config}) => {
    return (gatewayExpressApp) => {

        const middlewares = [
            bodyParser.urlencoded({extended: true}),
            bodyParser.json(),
        ];

        gatewayExpressApp.post(config.autenticacao.contextPath, middlewares, (req, res) => {

            if (req.headers.origin) {

                res.setHeader("Access-Control-Allow-Origin", req.headers.origin);
                res.setHeader("Access-Control-Allow-Credentials", "true");
            }

            let {dsLogin, dsSenha, idEmpresa, dsToken} = req.body;

            let dsHostacesso = getSubdomain(req.headers.host);

            let autenticacao;

            identificaUsuario(dsLogin, dsSenha, dsHostacesso, dsToken, config).then((response) => {

                autenticacao = response;

                return identificaEmpresa(autenticacao, idEmpresa, config);

            }).then((idEmpresaLogada) => {

                idEmpresa = idEmpresaLogada;

                return validaAcessoEmpresa(autenticacao, idEmpresa, config);

            }).then((temAcesso) => {

                if (temAcesso) {

                    return autenticacaoRequest.geraToken(dsLogin, dsSenha, dsHostacesso, idEmpresa,
                        dsToken, config.url.geracaoToken);

                } else {

                    throw Error("empresa.acesso.negado");

                }

            }).then((response) => {

                return res.send(response.body);

            }).catch((err) => {

                let errorMensage = err.error || err.message;

                return res.status(401).send({
                    error: errorMensage
                });

            });

        });

    };
};

const validaAcessoEmpresa = (autenticacao, idEmpresaLogada, config) => {

    return infoRequest.validaAcessoEmpresa(autenticacao, idEmpresaLogada, config.url.acessoEmpresa);

};

const identificaEmpresa = (autenticacao, idEmpresa, config) => {

    return new Promise((resolve, reject) => {

        if (idEmpresa) {

            return resolve(idEmpresa);

        } else if (autenticacao.idEmpresa) {

            return resolve(autenticacao.idEmpresa);

        } else if (autenticacao.id_empresa) {

            return resolve(autenticacao.id_empresa);

        } else {

            infoRequest.getInformacaoResumida(autenticacao, config.url.info).then((response) => {

                let info = response.body;

                return resolve(info.data.idEmpresaPadrao);

            }).catch((err) => {

                return reject(err);

            });

        }

    });

};

const identificaUsuario = (dsLogin, dsSenha, dsHostacesso, dsToken, config) => {

    return new Promise((resolve, reject) => {

        if (dsLogin && dsSenha) {

            autenticacaoRequest.autenticaUsuario(dsLogin, dsSenha, dsHostacesso, config.url.autenticacao).then((response) => {

                return resolve(response.body);

            }).catch((err) => {

                return reject(err);

            });

        } else {

            return resolve(jwt.decode(dsToken));

        }

    });

};

// TODO - exportar no NPM do odin-node
const getSubdomain = (host) => {

    var parts = host.split(".");
    if (parts[0] === 'www') {
        return parts[1];
    } else {
        return parts[0];
    }
};